package kjava.io;

import java.io.*;
import java.util.*;
import javax.microedition.io.*;
import javax.microedition.io.file.*;
import jsystem.*;



public class File{
	String fname = "";
	String dirname = ""; //JSystem.getCurrDirname();
    private final static String MEGA_ROOT = "/";

	public File(String fname){
		int dirIndex = fname.lastIndexOf('/');
  		if(dirIndex >= 0 && dirIndex < fname.length()-1){
			dirname = fname.substring(0, dirIndex+1);
			fname = fname.substring(dirIndex+1);
		}
      	this.fname = fname;
 	}

   public File(File dir, String name){
      this(name);
	  this.dirname = dir.fname;
   }

   public boolean isDirectory(){
      return fname.charAt(fname.length()-1) == '/';
   }

   public String getCanonicalPath() throws IOException{
      return dirname+fname;
   }

   public String getAbsolutePath() {
      return dirname+fname;
   }
   
   
	public String getName(){
		if(isDirectory()){
			String s = fname.substring(0,fname.length()-1);
			int idx = s.lastIndexOf('/');
			if(idx>=0)
				return s.substring(idx+1);
			else
				return s;
		}
   		return fname;
	}
	
	public boolean delete(){
		String path=dirname+fname;
		if(path.startsWith("res/"))
			return false;
		if(path.startsWith("vfs/")){
			try{
				JSystem.vfs.delete(fname);
				return true;
			}catch(Exception e){
				return false;
			}
		}else{
			try{
				FileConnection fc = (FileConnection)
                	Connector.open("file://localhost/" + path);
				fc.delete();
				return true;
			}catch(Exception e){
				return false;
			}
		}				
	}
	
	public boolean renameTo(String newName){
		String path=dirname+fname;
		if(path.startsWith("res/"))
			return false;
		if(path.startsWith("vfs/")){
			try{
				JSystem.vfs.rename(fname, newName);
				return true;
			}catch(Exception e){
				return false;
			}
		}else{
			try{
				FileConnection fc = (FileConnection)
                	Connector.open("file://localhost/" + path);
				fc.rename(newName);
				return true;
			}catch(Exception e){
				return false;
			}
		}				
	}
	
   
   	public boolean exists(){
		String p = getParent();
		if(p==null)
			return true;
		String[] dirlist = (new File(p)).list();
		if(dirlist==null || dirlist.length==0)
			return false;
		String path = dirname+fname;
		if(p.equals(MEGA_ROOT))
			p="";
		for(int i=0; i<dirlist.length; i++){
			if(path.equals(p+dirlist[i]))
				return true;
		}
		return false;
   	}
	
	public String getParent(){
		if(fname.equals(MEGA_ROOT))
			return null;
		if(!dirname.equals(""))
			return dirname;
		String p = fname.substring(0,fname.length()-1); // Strip '/'
		int idx = p.lastIndexOf('/');
		if( idx >= 0 )
			return p.substring(0, idx+1);
		else
			return MEGA_ROOT;
	}


   public String[] list(){
   		if(!isDirectory())
			return null;
		Enumeration e=null;
		String path = dirname+fname;
    	try {
   			if (MEGA_ROOT.equals(path)) {
    			e = FileSystemRegistry.listRoots();
			}else if("vfs/".equals(path)){
				e = JSystem.vfs.getFileList();
			}else if(path.startsWith("res/") ){
				// Read directory listing from File dir
				Vector flist = new Vector();
				try{				
					BufferedReader br = new BufferedReader(new FileReader(flist.getClass().getResourceAsStream(
								 		path.substring("res".length())+".dir" )));
					String p;
					while( (p=br.readLine()) != null){
						flist.addElement(p);
					}
					br.close();
				}catch(Exception e2){
				}
				e = flist.elements();
   			} else {
  				FileConnection currDir =  (FileConnection)Connector.open("file://localhost/" + 
                            path);
    			e = currDir.list();
				currDir.close();
    		}

			Vector fileList = new Vector();	
  			while (e!=null && e.hasMoreElements()) {
				fileList.addElement(e.nextElement());
			}
			if (MEGA_ROOT.equals(path)){
				fileList.addElement("vfs/");
				fileList.addElement("res/");
			}
			String[] files = new String[fileList.size()];
			fileList.copyInto((Object[])files);
			return files;
			
        } catch (IOException ioe) {
            ioe.printStackTrace();
			
        }
		return null;
	}

   public boolean isAbsolute(){
      return true;
   }

   public boolean canRead(){
      try{
          FileInputStream f = new FileInputStream(dirname+fname);
          f.close();
          return true;
      }catch(Exception e){
          return false;
      }
   }

   public long lastModified(){
   	if(dirname.equals("vfs/")){
		return JSystem.vfs.lastModified();
	}else{
   		try {
   			FileConnection fc = (FileConnection)Connector.open("file://localhost/" 
                                + dirname+fname);
        	if (!fc.exists()) {
                fc.create();
        	}
			long l = fc.lastModified();
			fc.close();
			return l;
     	}catch(Exception e){
	 		return 0;
	 	}
	 }
   }
   
	public long length(){
   		if(dirname.equals("vfs/")){
			return JSystem.vfs.length(fname);
		}else{
			long length = 0;
			try {
   				FileConnection fc = (FileConnection)Connector.open("file://localhost/" 
                                + dirname+fname);			
        		if (fc.exists()) {
					length = fc.fileSize();
				}
				fc.close();
     		}catch(Exception e){
			}
			return length;
		}
   	}
	
	public boolean mkdir(){
		if(fname.startsWith("vfs/") || fname.startsWith("res/"))
			return false;
		try {
   			FileConnection fc = (FileConnection)Connector.open("file://localhost/" 
                                + dirname+fname);			
        	if (fc.exists()) {
				fc.close();
				return false;
			}
			fc.mkdir();
			fc.close();
			return true;
		}catch(Exception e){
		}
		return false;
	}
		

   public static final String separator = "/";
   public static final String pathSeparator = "/";

   public static final char pathSeparatorChar = '/';
   public static final char separatorChar = ':';


}
